/**
 * Package: developodoFW, framework implemented in typescript to develop frontend web app
 * File: pageController.ts
 * Description: it defines the Controller abstract class that must be imported and implemented by all pages of the app
 * How to use it:
 * The page controller must import main.ts file and implement this class.
 * The page controller contruct must have the two first lines:
 *        constructor() {
 *            super();
 *            this.init();
 *        }
 * The page controller must implement the method
 *        OnInit() {} that executes its command after page is loaded
 *        OnUnload(){} that executes its commands before leaving the page
 * The page controller can use binding tools:
 *      -OneWayBinding: controller variables are directly shown on GUI. If they chage, GUI changes. GUI and 
 *                        Controller are connected unidirectionally: C -> G
 *          A) How to define them:
 *                    In Controller class must be declared and inicializated as properties of class.
 *                    class myController extends pageController{
 *                        myOneWayVariable:string='Hello World';
 *                        ...
 *                        constructor(){
 *                        ...
 *                     In View File (HTML) must be attached using mustache syntax:
 *                        <div>{{myOneWayVariable}}</div>
 *          B) How to use them:
 *                      Just change the value of the variable and it the new value will be rendered on GUI.
 *      -DoubleWayBindng: controller variables are linked with input fields of GUI. If the variable changes,
 *                        then input value changes. If an user change the value of the input, directly the
 *                        controller variable has the new value. GUI and Controller are connected bidirectionally: C <-> G
 *          A) How to define them:
 *                    In Controller class must be declared and inicializated as properties of class. (see OneWayBinding).
 *                    But the @twoWayBinding decorator must be used. This decorator receives a parameter: the id of the input
 *                    field connected to the variable.
 *                    class myController extends pageController{
 *                        @twoWayBinding('idOfMyInput') 
 *                        myTwoWayVariable:string='Hello World';
 *                        ...
 *                        constructor(){
 *                        ...
 *                    In View File (HTML) must exist and input field with the selected id:
 *                       <input type="text" id="idOfMyInput" value="">
 *                    It can be attached using mustache syntax, as the OneWayBinding variabels:
 *                        {{myTwoWayVariable:}}
 *          B) How to use them:
 *                    Just change the value of the variable and the new value will be rendered on GUI.
 *                    Just change the input value and the variable value will change.
 *      -PropertyBinding: controller variables can be connected to properties of elements, such as class, disabled, id, name, style...
 *                        If, the variable changes, the property changes. GUI and Controller are connected unidirectionally: C -> G
 *          A) How to define them:
 *                    In Controller class must be declared and inicializated as properties of class. (see TwoWayBinding).
 *                    But the @propertyBinding decorator must be used. This decorator receives two parameter: the id of html element
 *                    and the name of the parameter to be binded connected to the variable.
 *                    class myController extends pageController{
 *                        @propertyBinding('idOfMyInput','disabled') 
 *                        mypropertyVariable:boolean=true;
 *                        ...
 *                        constructor(){
 *                        ...
 *                    In View File (HTML) must exist and input field with the selected id:
 *                       <input type="text" id="idOfMyInput" value="">
 *          B) How to use them:
 *                    Just change the value of the variable and the new value will be applied to the property of the selected element.
 *                    For example: this.mypropertyVariable=false makes the input becomes enabled
 * ----
 * HOW TO CREATE A NEW PAGE
 * 1) Copy de framework folder (developodoFW) and install the dependencies:
 *  npm install jquery --save      npm install popper.js --save             npm install bootstrap --save
 *  npm install webpack webpack-cli webpack-dev-server —save-dev            npm install -D ts-loader
 * 2) tsc init  -> and config tsconfig.json
 *    npm init -y   //create package.json      npm install (to make sure all modules and libraries are installed)
 *    Config webpack.config.json  (npm start -> to simulate or npx webpack -> to build the bundle js)
 *    @see tsconfig.json, package.json and webpack.config.json examples.
 * 3) Create a view file: html and in the body add the app tag. If you want to bind something must be inside this tag.
 *      <body>
 *        <app>
 *          <!-- here your html code -->
 *        </app>
 *      </app>
 *      <!-- include the bundle js build by webpack -->
 *      <script src='index.js'></script>
 * 4) Create a controller file: index.ts:
 *    Import main file of framework in this way:
 *        import { pageController, twoWayBinding,propertyBinding, $ } from "./developodoFW/main";
 *    Create a controller class as it is explained on top of this file.
 *        class IndexPage extends pageController { --- }
 *    Create an instance of the controller class
 *        let indexpage = new IndexPage();
 *    If you want to bind the methods of your controller class in the html code, just export the class (last lines):
 *      export = {
 *         indexpage
 *       };
 *        Now you can access from HTML:   <button onclick="developodo.indexpage.myMethod()"></button>
 *        But before, you have to configure webpack.config.json library:
 *          output: {
 *              filename: 'index.js',
 *              path: __dirname,
 *              library:'developodo'  //<-change it if you want to
 *          },
 * 5) To test your page execute: npm start (if tsconfig.json and webpackage.config.json are corrected)
 *        http://localhost:8080
 * 6) To build your bundle file JS: index.js, execute: npx webpack
 *  
 * @autor: Carlos Serrano Sánchez for Developodo
 * @see core.ts
 * @see appGUI.ts
 */
import {App,$,propertyBinding,twoWayBinding} from './core';

/**
 * Main class of controller, exteds the core class App for binding tools
 */
abstract class pageController extends App {
  constructor() {
    super();
    //Config the OnUnload method to be executed on leaving the page
    $(window).on("unload", () => {
      this.OnUnload();
    });
  }
  //This method is called in the constructor of controller page
  init() {
    this.binding(this);  //analize DOM to inject onewaybinding code
    this.OnInit();  //Execute the OnInit method at the beginning
  }
  abstract OnInit();
  abstract OnUnload();
}

export { pageController, propertyBinding,twoWayBinding,$ };
