/**
 * File: index.ts
 * Description: it creates the controller class from indexPage (index.html) and executes it
 * @autor: Carlos Serrano Sánchez for Developodo
 * @see developodoFW
 */

/**
* Importing core frameworks tools: 
* pageController -> all binding utilities and oninit and unload methods
* twoWayBinding -> decorator for two way binding
* propertyBinding -> decorator for properties binding
* $ -> jQuery tools
*/
import { pageController, twoWayBinding,propertyBinding, $ } from "./developodoFW/main";
/**
 * Importing config file as enviroment object
 */
import * as enviroment from './enviroment.json';
/**
 * Importing client for Restful Service
 */
import { clientRestful } from "./client";
/**
 * Importing Subject for example purpose
 */
import { Subject } from "rxjs";

/**
 * item interface represents our data model.
 * All data objects in database fit with this interface
 */
interface item {
  id: number;
  title: string;
  description: string;
}

/**
 * Every page has a file (this) with a class with the name {namePage}Page,
 * IndexPage in this case, that must extend the core class pageController.
 * This class is the main controller of our GUI(View): index.html in this case.
 */
class IndexPage extends pageController {

  /**
   * Every one way binding must be declared as attribute of the controller class
   * and must be initialized (IMPORTANT). That allows us to use mustache code to
   * insert this variables into View, for example <button>{{editaddButton}}</button>
   * becomes <button>Add</button> in this default case.
   */
  editaddButton: string = "Add";
  editaddTitle: string = "Adding";
  
  /**
   * If you want to make a double way binding you just have to use de Decorator 
   * @twoWayBinding and the parameter: id of the input field of GUI. Only it is allowed on 
   * input fields (IMPORTANT), so in the HTML must exist and input with the id name
   * defined, for exable title.
   * After the decorator, the variable binded must be declared and initialized (IMPORTANT).
   * This variable can be rendered on View using mustache sintax
   */
  @twoWayBinding("title")
  title: string = "";
  @twoWayBinding("description")
  description: string = "";
  @twoWayBinding("idItem")
  idItem: string = "";

  /**
   * If you want to make a property binding you just have to use de Decorator 
   * @propertyBindig and the parameters: id of the field of GUI and property to bind. 
   * In this case, the submit button inside modal
   */
  @propertyBinding(".modal button[type=submit]","disabled")
  buttonAcceptModalDisabled:boolean=false;
  
  
  /**
   * Other variables for controller
   * myService: instance of the client Restful Service (CRUD)
   * data: an observable over the list of items. It check if there is 
   *       changes to update the table on GUI.
   */
  myService: clientRestful = null;
  data: Subject<item[]> = new Subject<item[]>();

  /**
   * Constructor has always two instructions:
   *   a call to super (pageController) is needed in Typescrip.
   *   the call to init() method to start core tools (binding data)
   */
  constructor() {
    super();
    this.init();
  }

  /**
   * OnInit method is called automatically after contructor.
   * Put here everything to be executed on load page
   */
  OnInit() {
    //Instance of Client with the endPoint URL as parameter (defined in config file)
    this.myService = new clientRestful(enviroment.endPoint);
    
    //Managing changes on data to update de table on GUI
    this.data.subscribe(d => {
      this.fillTable(d);
    });

    //Load data from RestFull Service and update data variable, so table is 
    //automatically updated on GUI
    this.refresh();
  }
  /**
   * OnUnLoad method is called automatically on UnLoad the page.
   * Put here everything to be executed before exiting
   */
  OnUnload(){
    //Unsubscribe the data observable
    this.data.unsubscribe();
  }
  //All the rest of methods of this CONTROLLER

  /**
   * addData shows an custom modal (see HTML file) with some 
   * doublewaybinding variables: title, description and idItem and other
   * onewaybinding variables: editaddButton and editaddTitle
   */
  addData(): void {
    //let's empty input fields of modal form by doublewaybinding
    //see alternative mode without doublewaybinding
    this.title = ""; //$("#title").val("");
    this.description = ""; //$("#description").val("");
    this.idItem = ""; //$("#idItem").val("");

    //Lets enable accept button by propertyBinding
    this.buttonAcceptModalDisabled=false;  //$("#addModalButton").prop("disabled", false);
    //As this form is used for edit and add, we cancel previus action on submit 
    $("#addForm").unbind("submit");
    
    //Set forom action to add Item
    $("#addForm").submit(() => {
      //it disabled accept button to avoid double click and strage behaviors
      this.buttonAcceptModalDisabled=true;  //$("#addModalButton").prop("disabled", true);
      //Some doublewaybinding maginc
      let t2 = this.title; //$("#title").val();
      let d2 = this.description; //$("#description").val();
      if (t2 != "" && d2 != "") {
        this.addDataConfirmed(t2, d2); //it uses client of service
      } else {
        this.buttonAcceptModalDisabled=false;  //$("#addModalButton").prop("disabled", false);
        alert("Fill all fields");
      }

      event.preventDefault();
    });
    //Some onewayBinding magic to set the GUI messages
    this.editaddButton = "Add";
    this.editaddTitle = "Adding";
    //it shows the modal
    $("#addModal").modal("show");
  }
  /**
   * addDataConfirmed is called from accept button of modal shown by addData function.
   * it uses the client to connect to API RestFul and add an item
   * Then, it updates the GUI
   * @param t title of item to edit (received from modal: editData function)
   * @param d description of item to edit (received from modal: editData function)
   */
  addDataConfirmed(t: string, d: string):void {
    //it uses the loading component offered by our custom framework gui tools
    this.gui.showLoading();
    //it uses the createItem method of the client service than send and POST request to add an item
    this.myService.createItem(t, d).subscribe(r => {
      //it hides the modal
      $("#addModal").modal("hide");
      //it reloads the GUI
      this.refresh();
      //it uses the toast utility implemented by our custom framework (see documentation)
      this.gui.showToast("A new Item added ("+t+")");
    });
  }

  /**
   * editData shows an custom modal (see HTML file) with some 
   * doublewaybinding variables: title, description and idItem and other
   * onewaybinding variables: editaddButton and editaddTitle
   * @param i id of item to edit (read from data-id)
   * @param t title of item to edit (read from data-title)
   * @param d description of item to edit (read from data-description)
   */
  editData(i: string, t: string, d: string): void {
    //let's populate data to input fields of modal form by doublewaybinding
    //see alternative mode without doublewaybinding
    this.title = t; //$("#title").val(t);
    this.description = d; //$("#description").val(d);
    this.idItem = i; //$("#idItem").val(i);
    
    //Lets enable accept button by propertyBinding
    this.buttonAcceptModalDisabled=false;  //$("#addModalButton").prop("disabled", false);
    //As this form is used for edit and add, we cancel previus action on submit 
    $("#addForm").unbind("submit");
    //Set forom action to edit Item
    $("#addForm").submit(() => {
      //it disabled accept button to avoid double click and strage behaviors
      this.buttonAcceptModalDisabled=true;  //$("#addModalButton").prop("disabled", true);
      
      //Some doublewaybinding magic
      let t2 = this.title; //let t2=$("#title").val(t);
      let d2 = this.description; //let d2=$("#description").val(t);
      if (t2 != "" && d2 != "") {
        this.editDataConfirmed(i, t2, d2);  //it uses client of service
      } else {
        alert("Fill all fields");
        this.buttonAcceptModalDisabled=false;  //$("#addModalButton").prop("disabled", false);;
      }
      event.preventDefault();
    });
    //Some onewayBinding magic to set the GUI messages
    this.editaddButton = "Edit";
    this.editaddTitle = "Editing";
    //it shows the modal
    $("#addModal").modal("show");
  }
  /**
   * editDataConfirmed is called from accept button of modal shown by editData function.
   * it uses the client to connect to API RestFul and edit the item
   * Then, it updates the GUI
   * @param i id of item to edit (received from modal: editData function)
   * @param t title of item to edit (received from modal: editData function)
   * @param d description of item to edit (received from modal: editData function)
   */
  editDataConfirmed(i: string, t: string, d: string):void {
    //it uses the loading component offered by our custom framework gui tools
    this.gui.showLoading();
    //it uses the updateItem method of the client service than send and PUT request to update an item
    this.myService.updateItem(i, t, d).subscribe(r => {
      //it hides the modal
      $("#addModal").modal("hide");
      //it reloads the GUI
      this.refresh();
      //it uses the toast utility implemented by our custom framework (see documentation)
      this.gui.showToast("Item edited ("+t+")");
    });
  }

  /**
   * removeData shows a modal created by GUI framework tool (see documentation) with some 
   * @param i id of item to remove (read from data-id)
   * @param t title of item to remove (read from data-title)
   */
  removeData(i: string, t: string): void {
    //it creates a modal with parameters: title, content, function executed by accept button
    this.gui.showModal("Removing Item",
    "<h1>" + t + "</h1>",
    "Remove it?",
    ()=>{
      this.removeDataConfirmed(i);
    } );
  }
  /**
   * removeDataConfirmed is called by the accept button of modal shown by removeData function
   * @param i id of item to be removed
   */
  removeDataConfirmed(i: string):void {
    //it uses client service removeItem function to send a DELETE request to remove an item from database
    this.myService.removeItem(i).subscribe(r => {
      //it removes directly from DOM, so we don't need to refresh data table
      $("#data" + i).remove();
      //it uses GUI tools for UX
      this.gui.showToast("Item removed");
    });
  }

  /**
   * refresh method show loading component and call getItems method from service cliente module.
   * When the observable (return) gets data or is completed, the input is injected in the data observable to be
   * captured and managed. Then, it hides loading component.
   */
  refresh():void {
    this.gui.showLoading();
    this.myService.getItems().subscribe(d => {
      this.data.next(d);  //send data received from server to data observable
      this.gui.hideLoading();
    });
  }

  /**
   * 
   * @param d data received from data observable from server: array of items
   */
  fillTable(d:item[]):void{
    //it empties the data table: remove all the children (html elements)
    $("#data tbody").empty();

    //if theres is something to be shown
    if (d.length > 0) {
      d.forEach(row => {
        let tr =`<tr id="data${row.id}">
                    <th scope="row">${row.id}</th>
                    <td>${row.title}</td>
                    <td>${row.description}</td>
                    <td>
                      <button type="button" class="btn btn-info btn-edit" data-id="${row.id}" 
                                  data-title="${row.title}" data-description="${row.description}">E</button>
                      <button type="button" class="btn btn-danger btn-remove" data-id="${row.id}" 
                                  data-title="${row.title}">D</button>
                    </td>
                 </tr>`;
        $("#data tbody").append(tr);
      });

      //there shouldn't be any button with event listener, because we just emptied the data table,
      //but just in any case, it removes all event listeners from all buttons.
      $(".btn-remove").unbind("click");
      $(".btn-edit").unbind("click");

      //it adds event (click) listener to add and edit buttons
      $(".btn-remove").click(x => {
        this.removeData($(x.target).data("id"), $(x.target).data("title"));
      });
      $(".btn-edit").click(x => {
        this.editData(
          $(x.target).data("id"),
          $(x.target).data("title"),
          $(x.target).data("description")
        );
      });
    }
  }
}

//This is the most important action: we create our controller instance
let indexpage = new IndexPage();


//this code is only for demostration purposes
indexpage.gui.showToast("Welcome to our experimental framework");
indexpage.gui.showToast("Only for educational purposes");
setTimeout(()=>{
  indexpage.gui.showToast("It could be much better :)");
},2000)

//this code is only for demostration purposes
$('#addModal').on('hidden.bs.modal',  ()=> {
  indexpage.title=""; //doublewaybinding beauty. Some code to show how it works. See the footer
})

//IMPORTANT: we export our controller in order to make it available form GUI. See webpack.config.json. Sew how index.html uses it.
export = {
  indexpage
};
