import React from "react";
import {
  Text,
  Button,
  StyleSheet,
  View,
  ScrollView,
  CameraRoll,
  Image,
  Vibration,
  PermissionsAndroid,
  TouchableOpacity,
  GetPhotosParamType
} from "react-native";
import * as RNFS from "react-native-fs";
import Share from "react-native-share";
import { RNCamera } from "react-native-camera";

//defining props accepted from parent
export interface Props {
  frontCamera: boolean;
}
//defining variable state <- any change here makes a new render
interface State {
  npics: number;
  photos: any;
  camera: any;
  lastCursor: string;
}

export default class CameraReact extends React.Component<Props, State> {
  //References of a html or component element, in this case to Camera Component
  camera = React.createRef<RNCamera>();
  cameraNode: RNCamera;

  constructor(props: Props) {
    super(props); //mandatory
    this.state = { npics: 0, photos: [], camera: styles.hide,lastCursor:null }; //default value

    this.clickOnImage = this.clickOnImage.bind(this); //bind this to this methods
    this.takePicture = this.takePicture.bind(this);
  }

  //This is executed after render, so we can get the reference to the camera component (in HTML render)
  componentDidMount() {
    if (this.camera.current) {
      this.cameraNode = this.camera.current;
    }
  }

  //General method to request Permissions with custom message
  async requestPermission(perm: any, tit, msg) {
    try {
      const granted = await PermissionsAndroid.request(perm, {
        title: tit,
        message: msg,
        buttonNeutral: "Ask Me Later",
        buttonNegative: "Cancel",
        buttonPositive: "OK"
      });
      if (granted === PermissionsAndroid.RESULTS.GRANTED) {
        //only android, it has to be updated to IOS
        return true;
      } else {
        console.warn("Permission denied");
        return false;
      }
    } catch (err) {
      console.error(err);
      return false;
    }
  }
  
  /**
   * Arrow definition, interesting.
   * This method access to the external storage to get pics from cameral roll.
   * It uses state.lastCursor to keep the last image loaded, in order to allow recall this
   * method a get another 10 pics more
  */
  private _loadImages = async () => {
    try {
      Vibration.vibrate(1000);
      let auth = await this.requestPermission(
        PermissionsAndroid.PERMISSIONS.READ_EXTERNAL_STORAGE,
        "Load Data",
        "Let's see, this app needs to access to the external storage in order to get your pics, can it?"
      );
      if (auth) {
        let props:GetPhotosParamType={
          first: 10,
          assetType: "Photos"
        }
        if(this.state.lastCursor){
          props.after=this.state.lastCursor;
        }
        let pics = await CameraRoll.getPhotos(props);
        
        this.setState({ photos: pics.edges.concat(this.state.photos) }); //concat new pics before last ones
        this.setState({lastCursor:pics.page_info.end_cursor});  //next reload starts from the last
        return true;
      } else {
        return false;
      }
    } catch (err) {
      console.error(err);
      return false;
    }
  };
 
  //This method use styles to hide or show the camera. It is called from a button
  private _showCamera = () => {
    this.state.camera==styles.container?
    this.setState({ camera: styles.hide }):
    this.setState({ camera: styles.container })
  };

  /**
   * this method takes a pic from the camera and convert it automatically to base64.
   * Then it hides the camera and call share method (clickOnImage)
   */
  async takePicture() {
    try {
      if (this.cameraNode) {
        const hasCameraPermissions = await this.requestPermission(
          PermissionsAndroid.PERMISSIONS.CAMERA,
          "Take Pic",
          "Let's take some pics, can i?"
        );
        if (!hasCameraPermissions) return;
        const options = { quality: 0.5, base64: true, fixOrientation: true };
        let data = await this.cameraNode.takePictureAsync(options);
        this.setState({ camera: styles.hide });
        await this.clickOnImage(data.base64, true); //d.base has directyle de base64 code
      }
    } catch (err) {
      console.error(err);
      return false;
    }
  }
  /**
   * 
   * @param x uri of pic to share if base 64 is false or null. Base64 code if base 64 is true
   * @param base64 indicates if x is an uri or base64 code directly
   */
  async clickOnImage(x: string, base64?: boolean) {
    try {
      //if it's uri, it gets the image from storage and convert it to base64
      if (!base64) {
        x = await RNFS.readFile(x, "base64");  
      }

      let shareOptions = {
        title: "My React Sample",
        dialogTitle: "My React Sample",
        subject: "My React Sample",
        message: "This is a test",
        url: `data:image/png;base64,` + x
      };
      //Native component to share info
      Share.open(shareOptions);
      return true;
    } catch (err) {
      console.error(err);
      return false;
    }
  }

  render() {
    let whichCamera=RNCamera.Constants.Type.back;
    if(this.props.frontCamera){
      whichCamera=RNCamera.Constants.Type.front;
    }
    return (
      <View>
        <View style={this.state.camera}>
          <RNCamera
            ref={this.camera}
            style={styles.preview}
            type={whichCamera}
            /*flashMode={RNCamera.Constants.FlashMode.on}*/
            androidCameraPermissionOptions={{
              title: "Permission to use camera",
              message: "We need your permission to use your camera",
              buttonPositive: "Ok",
              buttonNegative: "Cancel"
            }}
            androidRecordAudioPermissionOptions={{
              title: "Permission to use audio recording",
              message: "We need your permission to use your audio",
              buttonPositive: "Ok",
              buttonNegative: "Cancel"
            }}
          />
          <View>
            <TouchableOpacity
              onPress={this.takePicture.bind(this)}
              style={styles.capture}
            >
              <Text style={{ fontSize: 14 }}> SNAP </Text>
            </TouchableOpacity>
          </View>
        </View>
        <Button
          color="#841500"
          title="Take a pic"
          onPress={() => this._showCamera()}
        />
        <Button
          color="#841584"
          title="Load Images"
          onPress={() => this._loadImages()}
        />

        <ScrollView contentContainerStyle={styles.root}>
          {this.state.photos.map((p, i) => {
            return (
              <TouchableOpacity
                key={i}
                activeOpacity={0.5}
                onPress={() => this.clickOnImage(p.node.image.uri)}
              >
                <Image
                  key={i}
                  style={{
                    width: 300,
                    height: 100,
                    margin: 10,
                    borderRadius: 5,
                    borderColor: "#AAA",
                    borderWidth: 1
                  }}
                  source={{ uri: p.node.image.uri }}
                />
              </TouchableOpacity>
            );
          })}
        </ScrollView>
      </View>
    );
  }
}

// custom styles
const styles = StyleSheet.create({
  root: {
    justifyContent: "center",
    alignItems: "center"
  },
  images: {
    width: 300,
    height: 100,
    margin: 10
  },
  container: {
    backgroundColor: "black",
    display: "flex"
  },
  preview: {
    justifyContent: "flex-end",
    alignItems: "center",
    height: 400
  },
  hide: {
    height: 0,
    display: "none"
  },
  capture: {
    backgroundColor: "#ff0",
    borderRadius: 5,
    padding: 15,
    paddingHorizontal: 20,
    alignSelf: "center",
    margin: 20
  }
});
