import { Component, OnInit, TemplateRef } from "@angular/core";
import * as ClassicEditor from "@ckeditor/ckeditor5-build-classic";
import { database } from "firebase";
//Better than import * as database from 'firebase'
import { BsModalService, BsModalRef } from "ngx-bootstrap/modal";


@Component({
  selector: "app-home",
  templateUrl: "./home.component.html",
  styleUrls: ["./home.component.css"]
})
export class HomeComponent implements OnInit {
  public Editor = ClassicEditor;  //Editor constructor,
  public WYSIWYG: any;  //Editor instance
  private db; //firebase database instance

  public isDisabled = true;  //flag enabled editor
  private cursor;  //current cursor position
  private typing: boolean = false;  //user is typing?
  private timerToUpdateKey;  //delay to update after typing
  private virtualDoc;  //code in WYSIWYG to check if it needs to be updated

  public receiving: boolean = false;   //it's recevingn new data from db
  public sending: boolean = false;   //it's sending new data to db
  public updating: boolean = false;  //it's updatind WYSIWYG content
  public document:string; //modal content

  modalRef: BsModalRef;  //reference to modal

  //modalService inyected
  constructor(private modalService: BsModalService) {}

  //at the beginning
  ngOnInit() {
    //nothing to do in this example
  }
  
  /**
   *  This funcion is executed when editor is ready. We get a reference to instance (WYSISYG) and set focus
   * and the default cursor position
   * @param editor
   */
  public onReady(editor) {
    editor.ui
      .getEditableElement()
      .parentElement.insertBefore(
        editor.ui.view.toolbar.element,
        editor.ui.getEditableElement()
      );

    this.WYSIWYG = editor;

    this.isDisabled = true;  //disabled editor
    this.typing = false;  //user is not typing
    this.db = database();  //we create database object
    //it gets init value from data base
    this.db
      .ref()
      .once("value")
      .then(d => {
        if(d!=null)
          this.WYSIWYG.setData(d.val());  //it updates content
        else
          this.WYSIWYG.setData("Hello from Angular, Type here and check the database");  //it updates content
        this.enableEditor();  //it enables editor

        this.WYSIWYG.editing.view.focus();  //set focus on editor
        this.cursor = this.WYSIWYG.model.document.selection.getFirstPosition();  //we get current position: 0
        this.listenChanges();  //it starts listener from changes on database
      });

      //nothing else to do, editor is disabled until previous promise is dispathed (then)
    
  }

  /**
   * This function is executed every time we write on editor. We update the realtime database with new content.
   * @param e event keyup
   */
  onKeyDown(e) {
    this.typing = true;
    clearTimeout(this.timerToUpdateKey);
    this.timerToUpdateKey = setTimeout(() => {
      this.typing = false;
      this.cursor = this.WYSIWYG.model.document.selection.getLastPosition();
      this.onChange(null);
    }, 400);  //delay to avoid unnecessary changes on database
  }

  /**
   * This function updates cursor position
   * @param e keypup event
   */
  onKeyUp(e) {
    this.cursor = this.WYSIWYG.model.document.selection.getLastPosition();
  }

  /**
   * This function creates an agent that observes the changes in database
   */
  listenChanges() {
    this.stopListenChanges();
    this.db.ref().on("value", d => {
      /**
       * Let's update data. Editor loses focus and cursor position
       */
      this.receiving = true;
      setTimeout(() => {
        this.receiving = false;
      }, 100);   //to turn off receiving icon after 0.1s
    
      if (d.val() != this.virtualDoc) {  //it only updates if database sends new content
        this.virtualDoc = d.val();
        this.WYSIWYG.setData(this.virtualDoc);
        
        this.updating = true;
        setTimeout(() => {
          this.updating = false;
        }, 100);//to turn off updating icon after 0.1s

        //Let's focus again
        this.WYSIWYG.editing.view.focus();

        /**
         * Update position to keep cursor where user was writing on
         * This method is made only for this example and has bugs
         */
        this.WYSIWYG.model.change(writer => {
          let posStart = null;
          //if (this.WYSIWYG.model.hasContent(this.cursor.parent)) {
          try {
            if (this.cursor.offset <= this.cursor.parent.maxOffset) {
              let posStart = this.WYSIWYG.model.createPositionAt(
                this.cursor.parent,
                this.cursor.offset
              );
              const range = writer.createRange(posStart);
              writer.setSelection(range, { fake: true });
            }
          } catch (err) {
            console.log(err);
          }
          // }
        });
      }
    });
  }
 
  /**
   * This function is executed on exiting
   */
  ngOnDestroy() {
    //it stops agents listening the database changes
    this.stopListenChanges();
  }
  /**
   * This function remove all agent listening on value changes
   */
  stopListenChanges() {
    this.db.ref().off("value");
  }
  /**
   * These functions change the value binded with editor to enable it
   */
  disableEditor() {
    this.isDisabled = true;
  }
  enableEditor() {
    this.isDisabled = false;
  }
  
  /**
   * This funcion is fired when any change occurs on editor. We don't use it
   * @param editor that fires de change event
   */
  public onChange(e) {
    //nothing to do here
    if (!this.typing) {
      if (this.virtualDoc != this.WYSIWYG.getData()) {
        
        this.sending = true;
        setTimeout(() => {
          this.sending = false;
        }, 100); //to turn off sending icon after 0.1s
        
        this.virtualDoc = this.WYSIWYG.getData();
        this.db.ref().set(this.virtualDoc, error => {
          if (error) {
            console.log(error);
          } else {
            //nothing to do if success
          }
        });
      }
    }
  }

  /**
   * This function is called from button on view (HTML). 
   * The reference to HTML modal is passed.
   */
  openModal(template: TemplateRef<any>) {
    this.document=this.WYSIWYG.getData();
    this.modalRef = this.modalService.show(template);
  }
}
