import { Injectable } from "@angular/core";
import {auth} from "firebase";
import { AngularFireAuth } from "@angular/fire/auth";
import {
  Router,
  CanActivate
} from "@angular/router";
import { User } from "../model/user";

//this service is created by root application injector (for all app modules)
@Injectable({
  providedIn: "root"
})
export class AuthService implements CanActivate {
  //keep in RAM user credentials (email if it's logged in, null if it's not logged in, see user model interface)
  public user: User;

  constructor(private authS: AngularFireAuth, private router: Router) {
    this.isLogged();  //check user at the beggining
  }

  async login(user: string, pass: string) {
    return new Promise(async (resolve, reject) => {
      try {
        //it uses Google auth service by user and password.
        const res = await this.authS.auth.signInWithEmailAndPassword(user, pass);
        this.user = res.user;
        resolve(true);
      } catch (err) {
        reject(err);
      }
    });
  }

  async loginGoogle() {
    return new Promise(async (resolve, reject) => {
      try {
        //it uses Google auth by google credentials with pop up window
        let provider = new auth.GoogleAuthProvider();
        let res = await this.authS.auth.signInWithPopup(provider);
        this.user = res.user;
        resolve(true);
      } catch (err) {
        reject(err);
      }
    });
  }

  /**
   * it returns true is user is logged in. 
   */
  public isLogged(): boolean {
    if (this.user!=null) {
      return true;  //it is already logged and stored in RAM (user variable)
    } else {
      //is it stored in cookies (we keep user logged in by default)?
      if (localStorage.getItem("user")) {
        this.user = JSON.parse(localStorage.getItem("user"));  //ok, let's store in variable
        return true;
      } else {
        //if not in cookies, is it logged in cloud (server keeps cookies too)?
        this.authS.authState.subscribe(user => {
          if (user) {
            this.user = user; //YES, let's store in cookies and in RAM (variable)
            localStorage.setItem("user", JSON.stringify(this.user));
            return true;
          } else {
            //it is definitely not logged id. Let's remove from cookies to avoid bugs
            localStorage.removeItem("user");
            return false;
          }
        });
      }
    }
  }

  /**
   * Sign out in Cloud, in Cookies and in RAM. Then, user is redirected to login page.
   */
  public async logOut() {
    try {
      await this.authS.auth.signOut();  //next line executed when is loggout in cloud
      this.user = null;
      localStorage.removeItem("user");
      console.log("everyThing is removed");
      this.router.navigate(['login']);
    } catch (err) {
      console.log(err);
    }
  }

  /**
   * Router Guard. This method is called from routing module. 
   * If it returns true, then the page that user is trying to visit will be recheable.
   * The page won't be loaded if it returns false
   */
  public canActivate(): boolean {
    if (!this.isLogged()) {
      //this avoid bugs if user enters the url http//localhost:4200/home directly. 
      //The page wouldn't be recheable (thanks to routing module and canActivate call), but a blank 
      //page'd be loaded. To avoid this, we make sure to redirect user to login page.
      this.router.navigate(['login']);  
      return false;
    }
    return true;
  }

  /**
   * we don't use other type of Router Guard, there are here only for demostration
   */
  public canActivateChild(): boolean {
    return true;
  }
  public canDeactivate(): boolean {
    return true;
  }
  public canLoad(): boolean {
    return true;
  }
  public resolve(): boolean {
    return true;
  }
}
