import * as express from 'express';
import * as socketIo from 'socket.io';
import { eventsName } from './commons/eventsName';
import { Player } from './model/Player';
import { Score } from './model/Score';
import { Treasure } from './model/Treasure';
import { createServer, Server } from 'http';
var cors = require('cors');

interface IDictionary {
  [key: string]: Player;
};

export class shipFighterServer {
  // About the game
  players:IDictionary; //let a associative array
  treasure:Treasure;
  score:Score;
  static readonly horizontalSize = 800;
  static readonly verticalSize = 600;
  

  // About the server
  public static readonly PORT: number = 8080;
  private _app: express.Application;
  private server: Server;
  private io: SocketIO.Server;
  private port: string | number;

  constructor () {
    // setting web server
    this._app = express();
    this.port = process.env.PORT || shipFighterServer.PORT;  // for deploying
    this._app.use(cors());
    this._app.options('*', cors());

    /** web server for heroku */
    this._app.use("/dist",express.static('dist'));
    this._app.use("/assets",express.static('assets'));
    //told the server to serve the  index.html file as the root page.
    this._app.get('/', (req, res)=> {
      res.sendFile(__dirname + '/index.html');
    });

    this.server = createServer(this._app);
    
    // setting game variables
    this.initGame();

    // setting sockets over http server
    this.initSocket();
    
    // let's listen
    this.listen();
  }

  private initSocket (): void {
    this.io = socketIo(this.server);
  }

  private listen (): void {
    this.server.listen(this.port, () => {
      console.log('Running server on port %s', this.port);
    });

    //everthing starts here. When a player connects
    this.io.on(eventsName.CONNECTION, (socket: any) => {
      console.log('Connected client on port %s.', this.port);

      //when a player connects
      //1) add it to players array
      this.players[socket.id]={
          playerId:socket.id,
          rotation:0,
          x:this.getRandomH(),
          y:this.getRandomV(),
          team: Math.floor(Math.random()*2)==0?"red":"blue" // not well balanced, but easy
      }
      //2) send all players to this new player
      socket.emit(eventsName.CURRENTPLAYERS,this.players);
      //3) send treasure's position to this new player
      socket.emit(eventsName.TREASURELOCATION,this.treasure);
      //4) and, of course, send the scores to this new player
      socket.emit(eventsName.SCOREUPDATE,this.score);

      //5) send this new player to the rest of players
      socket.broadcast.emit(eventsName.NEWPLAYER,this.players[socket.id]);

      // when this player moves
      socket.on(eventsName.PLAYERMOVEMENT, (movementData:any) => {
        this.players[socket.id].x=movementData.x;
        this.players[socket.id].y=movementData.y;
        this.players[socket.id].rotation=movementData.rotation;
        // tell to other players that this player moved
        socket.broadcast.emit(eventsName.PLAYERMOVEMENT,this.players[socket.id]);
      });

      // when this player catches the treasure
      socket.on(eventsName.TREASURECOLLECTED, () => {
        // update the score variable on server
        this.players[socket.id].team === 'red'? (this.score.red+=10) : (this.score.blue+=10) ;
        // move the treasure to a new position
        this.treasure.x = this.getRandomH();
        this.treasure.y = this.getRandomV();
        // send new position to all players
        this.io.emit(eventsName.TREASURELOCATION,this.treasure);
        // send new score to all players
        this.io.emit(eventsName.SCOREUPDATE,this.score);
      });     

      // when this player disconnects
      socket.on(eventsName.DISCONNECT, () => {
        console.log('Client '+socket.id+'disconnected');
        //1) remove it from players array
        delete this.players[socket.id];
        //2) sent to the rest of players this situation
        this.io.emit(eventsName.DISCONNECT,socket.id);
        //if any player left, reset score
        if(Object.keys(this.players).length==0){
          this.score={
            blue:0,
            red:0
          }
        }
      });
    
    
    });
  }

  get app (): express.Application {
    return this._app;
  }

  initGame(){
      this.players={};
      this.score={
          blue:0,
          red:0
      }
      this.treasure={
          x: this.getRandomH(),
          y: this.getRandomV()
      }
  }
  getRandomH():number{
    return Math.floor(Math.random()*(shipFighterServer.horizontalSize-100)+50);
  }
  getRandomV():number{
    return Math.floor(Math.random()*(shipFighterServer.verticalSize-100)+50);

  }
}