import "phaser";
import {eventsName} from "../src/commons/eventsName"
import * as io from "socket.io-client";

export class GameScene extends Phaser.Scene {
  cursors: any;
  ship: any;
  treasure: Phaser.Physics.Arcade.Image;
  otherPlayers: any;
  socket: any;
  blueScoreText: any;
  redScoreText: any;
  client: SocketIO.Client;
  logo: any;

  constructor() {
    super({ key: "GameScene" }); //name scene
  }
  /**
   * It is called when scene starts
   * @param params
   */
  init(params): void {
    // TODO
    console.log("Scene started");
  }
  /**
   * It is called before the objects are created.
   * It contains loading assets and cacheable objects
   */
  preload(): void {
    // TODO
    this.logo=this.load.image('logo', './assets/logo.png');
    
    console.log("preload");
    this.load.image("ship", "./assets/me.png");
    this.load.image("otherPlayer", "./assets/enemy.png");
    this.load.image("treasure", "./assets/treasure.png");
  }
  /**
   * It is called when all assets are loaded. Normally, it creates the objects
   */
  create(): void {
    //this.logo=this.add.image(400, 300, 'logo');
    this.socket = io("http://localhost:8080"); //conecting to the same server?
    //this.socket = io();
    //We create a physics group for all the rest of player. The simplier way to
    //detect collisions...
    this.otherPlayers = this.physics.add.group();
    // We will handle player input by using Phaser’s built-in keyboard manager.
    this.cursors = this.input.keyboard.createCursorKeys();

    this.socket.on(eventsName.CURRENTPLAYERS, (players) => {
      //When this function is called, we loop through each of the players
      //and we check to see if that player’s id matches the current player’s socket id.

      //To loop through the players, we use  Object.keys() to create an array of all the keys
      //in the Object that is passed in. With the array that is returned we use the
      //forEach() method to loop through each item in the array.
      Object.keys(players).forEach((id) => {
        if (players[id].playerId === this.socket.id) {
          //Lastly, we called the  addPlayer() function and passed it the current player’s
          //information, and a reference to the current scene.
          this.addPlayer(players[id]);
        } else {
          //We updated the function that is called when the  currentPlayers event is emitted
          //to now call the  addOtherPlayers function when looping through the players object
          //if that player is not the current player.
          this.addOtherPlayers(players[id]);
        }
      });
    });

    this.socket.on(eventsName.NEWPLAYER, (playerInfo) => {
      //When the  newPlayer event is fired, we call the  addOtherPlayers function
      //to add that new player to our game.
      this.addOtherPlayers(playerInfo);
    });

    //When the  disconnect event is fired, we take that player’s id and we remove
    //that player’s ship from the game. We do this by calling the  getChildren() method on our
    //otherPlayers group. The  getChildren() method will return an array of all the game objects
    //that are in that group, and from there we use the  forEach() method to loop through that array.
    this.socket.on(eventsName.DISCONNECT, (playerId) => {
      this.otherPlayers.getChildren().forEach((otherPlayer) => {
        if (playerId === otherPlayer.playerId) {
          //Lastly, we use the  destroy() method to remove that game object from the game.
          otherPlayer.destroy();
        }
      });

      //we will need to update the client side code to listen for this new event,
      //and when this event is emitted
      this.socket.on(eventsName.PLAYERMOVEMENT, (playerInfo) => {
        this.otherPlayers.getChildren().forEach((otherPlayer) => {
          if (playerInfo.playerId === otherPlayer.playerId) {
            otherPlayer.setRotation(playerInfo.rotation);
            otherPlayer.setPosition(playerInfo.x, playerInfo.y);
          }
        });
      });
    });

    //In order to display the two teams scores, we will use Phaser’s Text Game Object.
    //We created two new text game objects by calling  this.add.text().
    //When we created these two objects, we passed the location of where the object would be placed,
    //the default text of the object, and the font size and fill that would be used for the text object.
    this.blueScoreText = this.add.text(16, 16, "", {
      fontSize: "32px",
      fill: "#4444FF"
    });
    this.redScoreText = this.add.text(580, 16, "", {
      fontSize: "32px",
      fill: "#FF4444"
    });

    //When the  scoreUpdate event is received, we then update the text of the game objects
    //by calling the  setText() method, and we pass the team’s score to each object.
    this.socket.on(eventsName.SCOREUPDATE, (scores) => {
      this.blueScoreText.setText("Blue: " + scores.blue);
      this.redScoreText.setText("Red: " + scores.red);
    });

    //We check to see if the star object exists, and if it does, we destroy it
    //We add a new star game object to the player’s game, and we use the information
    //passed to the event to populate its location
    //Lastly, we added a check to see if the player’s ship and the star object are
    //overlapping, and if they are, then we emit the  starCollected event.
    //By calling  physics.add.overlap, Phaser will automatically check for the overlap
    //and run the provided function when there is an overlap.
    this.socket.on(eventsName.TREASURELOCATION, (treasureLocation) => {
      if (this.treasure) this.treasure.destroy();
      this.treasure = this.physics.add.image(
        //this.physics.add.image(
        treasureLocation.x,
        treasureLocation.y,
        "treasure"
      );
      this.physics.add.overlap(
        this.ship,
        this.treasure,
        ()=> {
          this.socket.emit(eventsName.TREASURECOLLECTED);
        },
        null,
        self
      );
    });

    this.logo.destroy();
  }

  addPlayer(playerInfo) {
    //Created our player’s ship by using the x and y coordinates
    //that we generated in our server code.

    //Instead of just using  self.add.image to create our player’s ship,
    //we used  self.physics.add.image in order to allow that game object to use the arcade physics.

    //We used  setOrigin() to set the origin of the game object to be in the middle
    //of the object instead of the top left. The reason we did this because,
    //when you rotate a game object, it will be rotated around the origin point.

    //We used  setDisplaySize() to change the size and scale of the game object.
    //Originally, our ship image was 106×80 px, which was a little big for our game.
    //After calling  setDisplaySize() the image is now 53×40 px in the game.

    this.ship = this.physics.add
      .image(playerInfo.x, playerInfo.y, "ship")
      .setOrigin(0.5, 0.5)
      .setDisplaySize(53, 40);

    //We used  setTint() to change the color of the ship game object,
    //and we choose the color depending on the team that was generated
    //when we created our player info on the server.

    if (playerInfo.team === "blue") {
      this.ship.setTint(0xaaaaff);
    } else {
      this.ship.setTint(0xffaaaa);
    }
    //Lastly, we used  setDrag,  setAngularDrag, and  setMaxVelocity to modify
    //how the game object reacts to the arcade physics.
    //Both  setDrag and  setAngularDrag are used to control the amount of resistance
    //the object will face when it is moving.
    //setMaxVelocity is used to control the max speed the game object can reach.

    this.ship.setDrag(100);
    this.ship.setAngularDrag(100);
    this.ship.setMaxVelocity(200);
  }

  //This code is very similar to the code we added in the  addPlayer() function.
  //The main difference is that we added the other player’s game object to our
  //otherPlayers group.
  addOtherPlayers(playerInfo) {
    const otherPlayer = this.add
      .sprite(playerInfo.x, playerInfo.y, "otherPlayer")
      .setOrigin(0.5, 0.5)
      .setDisplaySize(53, 40);
    if (playerInfo.team === "blue") {
      otherPlayer.setTint(0xaaaaff);
    } else {
      otherPlayer.setTint(0xffaaaa);
    }
    otherPlayer["playerId"] = playerInfo.playerId;
    this.otherPlayers.add(otherPlayer);
  }

  /**
   * It is create each frame or time. It contains all dymamic stuff.
   * @param time
   */
  update() {
      //We are checking if the left, right, or up keys are pressed down.

      if (this.ship) {
        //If the left or right key is pressed, then we update the player’s angular velocity
        //by calling  setAngularVelocity(). The angular velocity will allow the ship to rotate left and right.

        if (this.cursors.left.isDown) {
          this.ship.setAngularVelocity(-150);
        } else if (this.cursors.right.isDown) {
          this.ship.setAngularVelocity(150);
        } else {
          //if neither the left or right keys are pressed, then we reset the angular velocity back to 0.
          this.ship.setAngularVelocity(0);
        }

        //If the up key is pressed, then we update the ship’s velocity, otherwise, we set it to 0.
        if (this.cursors.up.isDown) {
          this.physics.velocityFromRotation(
            this.ship.rotation + 1.5,
            100,
            this.ship.body.acceleration
          );
        } else {
          this.ship.setAcceleration(0);
        }

        //Lastly, if the ship goes off screen we want it to appear on the other side of the screen.
        //We do this by calling  physics.world.wrap() and we pass it the game object we want to wrap and an offset.
        this.physics.world.wrap(this.ship, 5);

        //We created three new variables and use them to store information about the player.
        // emit player movement
        let x = this.ship.x;
        let y = this.ship.y;
        let r = this.ship.rotation;

        //We then check to see if the player’s rotation or position has changed by comparing
        //these variables to the player’s previous rotation and position. If the player’s
        //position or rotation has changed, then we emit a new event called  playerMovement
        //and pass it the player’s information.

        if (
          this.ship.oldPosition &&
          (x !== this.ship.oldPosition.x ||
            y !== this.ship.oldPosition.y ||
            r !== this.ship.oldPosition.rotation)
        ) {
          this.socket.emit(eventsName.PLAYERMOVEMENT, {
            x: this.ship.x,
            y: this.ship.y,
            rotation: this.ship.rotation
          });
        }

        // save old position data
        //Lastly, we store the player’s current rotation and position.
        this.ship.oldPosition = {
          x: this.ship.x,
          y: this.ship.y,
          rotation: this.ship.rotation
        };
      }
    }
}
